@@ -45,9 +45,9 @@
 #'
 #'     \code{:q} is the command to quit.
 #'
-#'     Both parties must supply the same argument for 'auth', otherwise the
-#'     party trying to connect will receive an 'authentication error' and be
-#'     disconnected immediately.
+#'     Both parties must supply the same argument for \sQuote{auth}, otherwise
+#'     the party trying to connect will receive an \sQuote{authentication error}
+#'     and be immediately disconnected.
 #'
 #' @export
 #'
@@ -101,6 +101,8 @@ messenger <- function(url, auth = NULL) {
         cat(sprintf("%*s > %s\n", nchar(data), "", format.POSIXct(Sys.time())), file = stdout())
   }
 
+  if (!interactive()) later::run_now()
+
 }
 
 # nocov end
