@@ -99,7 +99,7 @@ nanotestaio(msg <- n1$recv_aio(mode = "complex", timeout = 500))
 nanotestn(stop_aio(msg))
 nanotestn(stop_aio(n))
 nanotest(identical(call_aio(msg), msg))
-nanotesterr(msg$data, "active")
+nanotest(is_error_value(msg$data))
 nanotest(identical(call_aio(n), n))
 nanotestaio(sraio <- n$send_aio(as.raw(0L), mode = "r", timeout = 500))
 nanotestaio(rraio <- n1$recv_aio(mode = 1L, timeout = 500))
@@ -204,7 +204,6 @@ nanotestaio(msg <- recv_aio(ctx, mode = 6L, timeout = 500))
 nanotest(is.logical(call_aio(msg)$data))
 nanotestaio(err <- send_aio(ctx, msg[["data"]], mode = "next"))
 nanotestn(stop_aio(err))
-nanotesterr(err$result, "active")
 nanotestaio(err <- send_aio(ctx, "test"))
 nanotest(is_error_value(call_aio(err)$result))
 nanotest(is_error_value(call_aio_(err)$result))
@@ -365,7 +364,7 @@ nanotestp(put1)
 nanotest(is.integer(call_aio_(put1)$status))
 nanotestnn(put1$headers)
 nanotestnn(put1$data)
-nanotest(is.null(stop_aio(put1)))
+nanotestn(stop_aio(put1))
 haio <- ncurl_aio("https://i.i")
 nanotest(is_error_value(call_aio(haio)$data))
 nanotestp(haio$data)
@@ -396,7 +395,7 @@ nanotesterr(stream(listen = "errorValue3", tls = "wrong"), "valid TLS")
 nanotesterr(stream(), "specify")
 
 nanotest(is.character(ver <- nng_version()) && length(ver) == 2L)
-nanotest(nng_error(8L) == "Try again")
+nanotest(nng_error(8L) == "8 | Try again")
 nanotest(is_nul_byte(as.raw(0L)))
 nanotest(!is_nul_byte(NULL))
 nanotest(!is_error_value(1L))
@@ -518,6 +517,7 @@ nanotestp(unres)
 nanotest(identical(call_aio("a"), "a"))
 nanotest(identical(call_aio_("a"), "a"))
 nanotestn(stop_aio("a"))
+nanotest(is.environment(set_promise_context(new.env(), new.env())))
 
 nanotest(base64enc("test") == "dGVzdA==")
 nanotest(base64dec(base64enc("test")) == "test")
@@ -555,3 +555,7 @@ nanotestw(dial(s, url = "tls+tcp://.", tls = tls, error = FALSE) > 0)
 nanotestw(listen(s, url = "tls+tcp://.", tls = tls, error = FALSE) > 0)
 nanotestz(close(s1))
 nanotestz(close(s))
+rm(list = ls())
+gc()
+Sys.sleep(1L)
+.Call(nanonext:::rnng_fini)
